@@ -1,4 +1,5 @@
 
+#include "Python.h"
 #include "config.h"
 
 #ifdef HAVE_ALLOCA_H
@@ -11,7 +12,6 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#include "Python.h"
 #include "structmember.h"
 #include "nls.h"
 #include "newt.h"
@@ -317,7 +317,15 @@ static PyMethodDef widgetMethods[] = {
 };
 
 static PyMemberDef widget_members[] = {
-        { "key" , T_INT, offsetof(snackWidget, co), 0, NULL },
+        { "key",
+#if SIZEOF_VOID_P == SIZEOF_LONG
+		T_LONG,
+#elif SIZEOF_VOID_P == SIZEOF_LONG_LONG
+		T_LONGLONG,
+#else
+#error unsupported pointer size
+#endif
+		offsetof(snackWidget, co), 0, NULL },
         { "entryValue", T_STRING, offsetof(snackWidget, apointer), 0, NULL },
 	{ NULL }
 };
@@ -1019,8 +1027,14 @@ static PyObject * formRun(snackForm * s, PyObject * args) {
 	return Py_BuildValue("(si)", "timer", 0);
     else if (result.reason == NEWT_EXIT_FDREADY)
 	return Py_BuildValue("(si)", "fdready", result.u.watch);
+    else if (result.reason == NEWT_EXIT_COMPONENT)
+#if SIZEOF_VOID_P <= SIZEOF_LONG
+	return Py_BuildValue("(sl)", "widget", (long)result.u.co);
+#else
+	return Py_BuildValue("(sL)", "widget", (long long)result.u.co);
+#endif
     else
-	return Py_BuildValue("(si)", "widget", result.u.co);
+	return Py_BuildValue("(si)", "error", 0);
 }
 
 static PyObject * formHotKey(snackForm * s, PyObject * args) {
@@ -1455,6 +1469,7 @@ MOD_INIT(_snack)
     setitemstring_decref(d, "FORM_EXIT_WIDGET", PyString_FromString("widget"));
     setitemstring_decref(d, "FORM_EXIT_TIMER", PyString_FromString("timer"));
     setitemstring_decref(d, "FORM_EXIT_FDREADY", PyString_FromString("fdready"));
+    setitemstring_decref(d, "FORM_EXIT_ERROR", PyString_FromString("error"));
 
     setitemstring_decref(d, "KEY_TAB", PyInt_FromLong(NEWT_KEY_TAB));
     setitemstring_decref(d, "KEY_ENTER", PyInt_FromLong(NEWT_KEY_ENTER));
